#!/usr/bin/env python
#-*- coding:utf-8 -*-

#    Pybik -- A 3 dimensional magic cube game.
#    Copyright (C) 2009              B. Clausius
#    Copyright (C) 1998, 2003--2004  John Darrington
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License,  or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import sys
import getopt

import config


#@debug
def import_(pure_python):
    global cube, ui, widget_set, select_
    
    if not pure_python:
        try:
            import drwBlock_c
            sys.modules['pybiklib.drwBlock'] = drwBlock_c
            sys.modules['pybiklib.cube'] = drwBlock_c
        except ImportError:
            print _('Module drwBlock_c not found, using fallback. Rendering will be slow.')

    import cube
    import ui
    import widget_set
    import select_


def main(argv):
    # process arguments specific to this program
    opt_vals = app_opts (argv[1:], frameQty=2, solved=False, cube_dimension=3,
                                    pure_python=False, debug_level=0)
    import_(opt_vals['pure_python'])
    run(argv, opt_vals['frameQty'], opt_vals['cube_dimension'], opt_vals['solved'])


def run(argv, frameQty, cube_dimension, solved):
    ui.frameQty = frameQty

    # Create the top level widget --- that is,  the main window which everything
    #   goes in
    main_window = widget_set.MainWindow(argv)

    # create the cube
    cube.set_cube_dimension(cube_dimension)

    # If a solved cube has not been requested,  then do some random
    #   moves on it
    if not solved:
        cube.cube_randomize()

    # initialise the selection mechanism
    #select_.initSelection (50,  1,  select_.selection_func)
    select_.initSelection (10,  1,  select_.selection_func)

    main_window.start_main_loop ()
    


# process the options we're interested in.  X resource overrides should
#have already been extracted
def app_opts (argv, **opt_vals):
    shortopts = "vhsz:a:"
    longopts = ["version","help","solved","size=","animation=",
                'pure-python','debug=']

    opts,unused_args = getopt.getopt(argv, shortopts, longopts)
    for c,optarg in opts:
        if c in ['-a', "--animation"]:      opt_vals['frameQty'] = int(optarg)
        elif c in ['-s', "--solved"]:       opt_vals['solved'] = True
        elif c in ['-z', "--size"]:         opt_vals['cube_dimension'] = int(optarg)
        elif c in ["--pure-python"]:        opt_vals['pure_python'] = True
        elif c in ["--debug"]:              opt_vals['debug_level'] = int(optarg)
        elif c in ['-h', "--help"]:
            print help_string
            exit (0)
        elif c in ['-v', "--version"]:
            print config.VERSION
            print config.copyleft_notice,
            exit (0)
        else:
            print _('unknown option:'), c
            print help_string
            exit (0)
    
    return opt_vals


help_string = _(
'''Usage: pybik [OPTION]...
  -h, --help         Show this help message
  -v, --version      Show version number
  -s, --solved       Start with the cube already solved
  -z, --size=N       Start with a N x N x N sized cube
  -a, --animation=N  Show N intermediate positions in animations
      --pure-python  Use python module for rendering (very slow)
      --debug=N      Enable debug output''')


if __name__ == '__main__':
    sys.exit(main(sys.argv))

