#-*- coding:utf-8 -*-

#    Pybik -- A 3 dimensional magic cube game.
#    Copyright (C) 2009  B. Clausius
#    Copyright (C) 2004  Dale Mellor
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License,  or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import sys

import config

import cube
import ui
import widget_set
import move_queue_m


# Data type to implement a singly-linked list of GtkItemFactoryEntry
# objects.
#FIXME GtkItemFactoryEntry is obsolet
class Menu_Item_List:
    def __init__(self):
        self.entry = self
        self.callback_data = ''
        self.next = None


# The head of a singly linked list of menu items that the plugins request
# as they are imported.
list_head = []


# When a script runs,  the first cube movement that is requested by the script
# must flush the move queue after the place of current insertion; further move
# requests must be appended to the end of the move queue. This variable is used
# to track the first request.
moved = False


# This function is called from the menu when the user makes a selection. The
# data is a string which was registered with the menu system and gives the name
# of a procedure to execute.
@debug
def run_script (func, application):
    global moved
    moved = False

    func()
    ui.request_play (None, application)


# Function callable from plugin which returns a structure
# reflecting the current state of the cube.
@debug
def gnubik_cube_state ():
    return cube.make_plugin_cube ()


# The first time a script makes a move on the cube,  the move_queue must be
# truncated to the current place,  and the place is marked so that the viewer
# can rewind the effects of the script. This script performs the necessary
# preparations.
def start_moves_if_first ():
    global moved
    if not moved:
        moved = 1
        ui.request_truncate_move_queue ()
        ui.request_mark_move_queue (None)


# Function which, when called from plugins, causes one
# side of the cube to rotate on-screen.
@debug
def gnubik_rotate_animated (list_):
    start_moves_if_first ()

    #print list_
    while list_:
        move = move_queue_m.MoveData()
        
        #move.axis = list_[0]
        #move.slice = list_[1]
        #move.dir = list_[2]
        move.axis = list_[0][0]
        move.slice = list_[0][1]
        move.dir = list_[0][2]
        
        ui.request_delayed_rotation (move)
        
        #list_ = list_[3:]
        list_ = list_[1:]

    return None


# Function allowing a script to apply all its moves in one go to the cube,
# without creating animations on the display.
@debug
def gnubik_run_moves ():
    ui.request_fast_forward (widget_set.main_application_window)
    return None


# Function to allow a plugin to display a message to the user.
@debug
def gnubik_error_dialog (message):
    msg = message
    widget_set.error_dialog (widget_set.main_application_window,  msg)
    return None


# Function to scan the named directory for all files with a .py extension, and import them.
@debug
def read_script_directory (dir_name):
    if os.path.isdir(dir_name):
        sys.path.insert(0, dir_name)
        for entry in os.listdir(dir_name):
            if ".py" == entry[-3:]:
                module_name = entry[:-3]
                #buffer = "%s/%s" % (dir_name, entry)
                import imp
                file_,path,desc = imp.find_module(module_name, [dir_name])
                try:
                    module = imp.load_module(module_name, file_, path, desc)
                    if DEBUG:
                        print 'import', module_name
                except SyntaxError, e:
                    print e
                except ImportError, e:
                    print e
                else:
                    for fname, func in plugin_funcs:
                        setattr(module, fname, func)
                    if hasattr(module,'scripts'):
                        for menu_name, menu_func in module.scripts:
                            list_head.append((menu_name, menu_func))
                finally:
                    if file_:
                        file_.close()
                

plugin_funcs = [
    ("gnubik_cube_state",       gnubik_cube_state),
    ("gnubik_rotate_animated",  gnubik_rotate_animated),
    ("gnubik_run_moves",        gnubik_run_moves),
    ("gnubik_error_dialog",     gnubik_error_dialog),
    ('random',                  lambda x:cube.rand.randrange(x)),
]

# This function initializes the plugins for us, and once the plugins have
# all been imported, it returns the requested menu structure to the caller.
@debug
def startup_plugins ():
    local_dir = "/.gnubik/scripts"

    # Import all files in .../share/pybik.

    #FIXME
    #read_script_directory (SCRIPTDIR)
    #read_script_directory (os.path.dirname(__file__)+'/../scripts')
    read_script_directory (config.path.plugindir)

    # Run all the user scripts in $(HOME)/.gnubik/scripts.
    home_dir = os.getenv ("HOME")
    if home_dir:
        read_script_directory (home_dir + local_dir)

    return list_head

